from policyengine_us.model_api import *


class non_qualified_dividend_income(Variable):
    value_type = float
    entity = Person
    label = "non-qualified dividend income"
    unit = USD
    definition_period = YEAR
    uprating = "calibration.gov.irs.soi.non_qualified_dividend_income"



from policyengine_us.model_api import *


class spm_unit_federal_tax_reported(Variable):
    value_type = float
    entity = SPMUnit
    label = "Federal income tax (reported"
    definition_period = YEAR
    unit = USD



from policyengine_us.model_api import *


class nm_non_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Mexico non-refundable credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.NM



from policyengine_us.model_api import *


class metered_gas_expense(Variable):
    value_type = float
    entity = SPMUnit
    label = "Metered gas expense"
    unit = USD
    definition_period = YEAR



from policyengine_us.model_api import *


class personal_property(Variable):
    value_type = float
    entity = Person
    label = "Personal property value"
    unit = USD
    definition_period = YEAR



from policyengine_us.model_api import *


class nd_ltcg_subtraction(Variable):
    value_type = float
    entity = TaxUnit
    label = "North Dakota long-term capital gains subtraction from federal taxable income"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2021-iit/form-nd-1-2021.pdf#page=1"  # line 7
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2021-iit/individual-income-tax-booklet-2021.pdf#page=14"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2022-iit/form-nd-1-2022.pdf#page=1"  # line 7
        "https://www.tax.nd.gov/sites/www/files/documents/forms/individual/2022-iit/2022-individual-income-tax-booklet.pdf#page=14"
    )
    defined_for = StateCode.ND

    def formula(tax_unit, period, parameters):
        ltcg = add(tax_unit, period, ["long_term_capital_gains"])
        stcg = add(tax_unit, period, ["short_term_capital_gains"])
        net_ltcg = min_(ltcg, ltcg + stcg)
        p = parameters(period).gov.states.nd.tax.income
        return net_ltcg * p.taxable_income.subtractions.ltcg_fraction



from policyengine_us.model_api import *


class ar_income_tax_before_non_refundable_credits_unit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Arkansas income tax before non refundable credits combined"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.dfa.arkansas.gov/images/uploads/incomeTaxOffice/2023_AR1000F_and_AR1000NR_Instructions.pdf"
        "https://www.dfa.arkansas.gov/images/uploads/incomeTaxOffice/2023_AR1000F_FullYearResidentIndividualIncomeTaxReturn.pdf"
    )
    defined_for = StateCode.AR

    def formula(tax_unit, period, parameters):
        filing_separately = tax_unit("ar_files_separately", period)
        itax_indiv = add(
            tax_unit,
            period,
            ["ar_income_tax_before_non_refundable_credits_indiv"],
        )
        itax_joint = add(
            tax_unit,
            period,
            ["ar_income_tax_before_non_refundable_credits_joint"],
        )

        return where(filing_separately, itax_indiv, itax_joint)



from policyengine_us.model_api import *


class wi_base_exemption(Variable):
    value_type = float
    entity = TaxUnit
    label = "Wisconsin base exemption"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.revenue.wi.gov/TaxForms2021/2021-Form1f.pdf"
        "https://www.revenue.wi.gov/TaxForms2021/2021-Form1-Inst.pdf"
        "https://www.revenue.wi.gov/TaxForms2022/2022-Form1f.pdf"
        "https://www.revenue.wi.gov/TaxForms2022/2022-Form1-Inst.pdf"
        "https://docs.legis.wisconsin.gov/misc/lfb/informational_papers/january_2023/0002_individual_income_tax_informational_paper_2.pdf"
    )
    defined_for = StateCode.WI

    def formula(tax_unit, period, parameters):
        # compute base exemption amount
        p = parameters(period).gov.states.wi.tax.income
        return tax_unit("exemptions_count", period) * p.exemption.base



from policyengine_us.model_api import *


class ca_income_tax_before_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "CA income tax before credits"
    defined_for = StateCode.CA
    unit = USD
    definition_period = YEAR
    reference = "https://www.ftb.ca.gov/forms/Search/Home/Confirmation"

    def formula(tax_unit, period, parameters):
        filing_status = tax_unit("filing_status", period)
        taxable_income = tax_unit("ca_taxable_income", period)
        p = parameters(period).gov.states.ca.tax.income.rates

        statuses = filing_status.possible_values

        return select(
            [
                filing_status == statuses.SINGLE,
                filing_status == statuses.SEPARATE,
                filing_status == statuses.JOINT,
                filing_status == statuses.SURVIVING_SPOUSE,
                filing_status == statuses.HEAD_OF_HOUSEHOLD,
            ],
            [
                p.single.calc(taxable_income),
                p.separate.calc(taxable_income),
                p.joint.calc(taxable_income),
                p.surviving_spouse.calc(taxable_income),
                p.head_of_household.calc(taxable_income),
            ],
        )



from policyengine_us.model_api import *


class ks_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "Kansas income tax"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.ksrevenue.gov/pdf/ip21.pdf"
        "https://www.ksrevenue.gov/pdf/ip22.pdf"
    )
    defined_for = StateCode.KS
    adds = ["ks_income_tax_before_refundable_credits"]
    subtracts = ["ks_refundable_credits"]



from policyengine_us.model_api import *


class tax_exempt_public_pension_income(Variable):
    value_type = float
    entity = Person
    label = "tax-exempt public pension income"
    unit = USD
    documentation = "Tax-exempt income from government employee pensions."
    definition_period = YEAR



from policyengine_us.model_api import *


class ms_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "Mississippi income tax"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MS

    def formula(tax_unit, period, parameters):
        tax_before_credits = tax_unit(
            "ms_income_tax_before_credits_unit", period
        )
        non_refundable_credits = tax_unit("ms_non_refundable_credits", period)
        return max_(tax_before_credits - non_refundable_credits, 0)



from policyengine_us.model_api import *


class mi_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Michigan refundable credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MI

    adds = "gov.states.mi.tax.income.credits.refundable"



from policyengine_us.model_api import *


class tax_unit_children(Variable):
    value_type = float
    entity = TaxUnit
    label = "Number of children in tax unit"
    unit = USD
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        return add(tax_unit, period, ["is_child"])



from policyengine_us.model_api import *


class mt_non_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "Montana refundable credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.MT

    adds = "gov.states.mt.tax.income.credits.non_refundable"



from policyengine_us.model_api import *


class ca_care_amount_if_eligible(Variable):
    value_type = float
    entity = Household
    definition_period = YEAR
    unit = USD
    label = "California CARE amount discounted"
    documentation = "California's CARE program provides this electricity discount to eligible households."
    reference = "https://www.cpuc.ca.gov/industries-and-topics/electrical-energy/electric-costs/care-fera-program"
    defined_for = StateCode.CA

    def formula(household, period, parameters):
        expense = add(household, period, ["pre_subsidy_electricity_expense"])
        p = parameters(period).gov.states.ca.cpuc.care
        return p.discount * expense



from policyengine_us.model_api import *


class al_interest_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Alabama investment interest"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://law.justia.com/codes/alabama/2022/title-40/chapter-18/article-1/section-40-18-15/"
        # Code of Alabama Section 40-18-15
        "https://www.revenue.alabama.gov/ultraviewer/viewer/basic_viewer/index.html?form=2023/01/22f40schabdc_blk.pdf#page=1"
        # 2022 Schedule A (Form 1040)
        "https://www.revenue.alabama.gov/ultraviewer/viewer/basic_viewer/index.html?form=2022/06/21f40schabdc_blk.pdf#page=1"
        # 2021 Schedule A (Form 1040)
    )
    defined_for = StateCode.AL

    adds = ["mortgage_interest", "investment_interest_expense"]



from policyengine_us.model_api import *


class dc_power(Variable):
    value_type = float
    entity = SPMUnit
    label = "DC Program on Work, Employment, and Responsibility (POWER)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://code.dccouncil.gov/us/dc/council/code/sections/4-205.52"
    )
    defined_for = "dc_power_eligible"

    def formula(spm_unit, period, parameters):
        standard_payment = spm_unit("dc_tanf_standard_payment", period)
        countable_income = spm_unit("dc_tanf_countable_income", period)
        return max_(standard_payment - countable_income, 0)
